package alg;

import java.util.*;
import java.io.*;

// Rect serves as a storage for the rectangle parameters
// Wheteher it is filled or hollow decides the function which paints it
class Rect{
int x0, x1, y0, y1, color, thick;
}


public class Editor {

static int width;     // of image
static int height;    // of image
static int [][] img;  // image itself, each pixel contains integer color id.

static Rect inRect = new Rect(); // surprisingly, one Rect is enough through
                                 // the entire program
static String foo;   // just auxiliary

// keep systematically [x0, y0] as upper left corner of the rectangle
static void normalize(Rect r){
  if (r.x0 > r.x1)  {r.x0 += r.x1; r.x1 = r.x0-r.x1; r.x0 -= r.x1; } // swap
  if (r.y0 > r.y1)  {r.y0 += r.y1; r.y1 = r.y0-r.y1; r.y0 -= r.y1; } // swap
}

static void fillRect(int x0, int y0, int x1, int y1, int color) {
  if ( ((x0 > width) && (y0 >= height)) || ((x1 < 0) && (y1 < 0)))
    return;
  for (int i = Math.max(y0, 0); i <= Math.min(y1, height-1); i++)
    for (int j = Math.max(x0, 0); j <= Math.min(x1, width-1); j++)
      img[i][j] = color;
}

static void hollowRect(Rect r) {
  if ((r.thick*2 >= r.x1-r.x0+1) || (r.thick*2 >= r.y1-r.y0+1))  // border too thick
    fillRect(r.x0, r.y0, r.x1, r.y1, r.color);
  else {     // some empty space inside the borders
    fillRect(r.x0, r.y0, r.x1, r.y0+r.thick-1, r.color); // upper border
    fillRect(r.x0, r.y0+r.thick, r.x0+r.thick-1, r.y1-r.thick, r.color); // left border
    fillRect(r.x1-r.thick+1, r.y0+r.thick, r.x1, r.y1-r.thick, r.color); // right border
    fillRect(r.x0, r.y1-r.thick+1, r.x1, r.y1, r.color); // bottom border
  }
}

static void readInRect(String inSg){
  StringTokenizer st = new StringTokenizer(inSg);
  foo = st.nextToken(); // skip the command symbol
  inRect.y0 = Integer.parseInt(st.nextToken());
  inRect.x0 = Integer.parseInt(st.nextToken());
  if (inSg.charAt(0) == 'F') {
    inRect.color = Integer.parseInt(st.nextToken());
    return;
  }
  inRect.y1 = Integer.parseInt(st.nextToken());
  inRect.x1 = Integer.parseInt(st.nextToken());
  if (inSg.charAt(0) == 'R') {
    inRect.color = Integer.parseInt(st.nextToken());
    return;
  }
  // hollow rect remains
  inRect.thick = Integer.parseInt(st.nextToken());
  inRect.color = Integer.parseInt(st.nextToken());

}

static void execCommand(String sgCommand) {
  readInRect(sgCommand);
  switch (sgCommand.charAt(0)) {
    case 'R' : {
        normalize(inRect);
        fillRect(inRect.x0, inRect.y0, inRect.x1, inRect.y1, inRect.color);
      break; }
    case 'H' : {
        normalize(inRect);
        hollowRect(inRect);
      break; }
    case 'F': {
        floodFill(inRect.x0, inRect.y0, inRect.color);
      break; }
 }
}

// Flood fill in its simplest incarnation starts in a given pixel (root node)
// and then considers the adjacent pixels to be children nodes in a (at most)
//quaternary tree. Thus the Breadth-First Search algorithm is applied and a
// queue must be used.

static int [] qx;  // x coord of the pixel in the queue
static int [] qy;  // y coord ...

static void floodFill(int x, int y, int newColor) {
 if ((x < 0) || (y < 0) || (x >=width) || (y >= height))
   return;
 int oldColor = img[y][x];
 if (oldColor == newColor)
   return;
 int head = 0;
 int tail = 0; // tail
 qx[tail] = x;
 qy[tail] = y;
 img[y][x] = newColor;
 while(head <= tail) { // while queue unempty
   x = qx[head];
   y = qy[head++];
   //listImg();
   //System.out.printf("%d %d\n",y, x );
   if ((y > 0) && (img[y-1][x] == oldColor)) {
     qx[++tail] = x;  qy[tail] = y-1;
     img[y-1][x] = newColor;
   }
   if ((x > 0) && (img[y][x-1] == oldColor)) {
     qx[++tail] = x-1;  qy[tail] = y;
     img[y][x-1] = newColor;
   }
   if ((y < height-1) && (img[y+1][x] == oldColor)) {
     qx[++tail] = x;  qy[tail] = y+1;
     img[y+1][x] = newColor;
   }
   if ((x < width-1) && (img[y][x+1] == oldColor)) {
     qx[++tail] = x+1;  qy[tail] = y;
     img[y][x+1] = newColor;
   }
 }
}

static void listColorCounts(){
  int [] ccount = new int[256];
  for (int i = 0; i < height; i++)
    for (int j = 0; j < width; j++)
      ccount[img[i][j]]++;
  System.out.printf("%d %d\n", 0, ccount[0]); // this is obligatory
  for (int i = 1; i < ccount.length; i++)
    if(ccount[i] > 0)
      System.out.printf("%d %d\n", i, ccount[i]);
}

public static void main(String[] args) throws IOException {

  BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
  String inputLine;
  StringTokenizer st = new StringTokenizer(br.readLine());
  height = Integer.parseInt(st.nextToken());
  width = Integer.parseInt(st.nextToken());

  img = new int [height][width];
  qx = new int [width*height];
  qy = new int [width*height];
  while (true) {
    inputLine = br.readLine();
    if (inputLine.charAt(0) == 'E')
      break;
    execCommand(inputLine);
  }
 listColorCounts();
}

}
